#!/bin/bash -e
# Note: must be bash; uses bash-specific tricks
#
# ******************************************************************************************************************
# This OpenVPN script does everything! It handles TUN and TAP interfaces,
# pushed configurations and DHCP leases. :)
#
# This is the "Down" version of the script, executed after the connection is
# closed.
#
# Created by: Nick Williams
# Modified by: Fabian Jaeger for Shimo
#
# ******************************************************************************************************************

##########################################################################################
# @param String message - The message to log
logMessage()
{
    echo "$(date '+%Y-%m-%d %T') *Shimo: "${@} >> "${SCRIPT_LOG_FILE}"
}

##########################################################################################
# @param String string - Content to trim
trim()
{
    echo ${@}
}

##########################################################################################
flushDNSCache()
{
    if ${ARG_FLUSH_DNS_CACHE} ; then
        set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors
        readonly OSVER="$(sw_vers | grep 'ProductVersion:' | grep -o '10\.[0-9]*')"
        set -e # We instruct bash that it CAN again fail on errors
        case "${OSVER}" in
            * )
                set +e # "grep" will return error status (1) if no matches are found, so don't fail on individual errors
                hands_off_ps="$( ps -ax | grep HandsOffDaemon | grep -v grep.HandsOffDaemon )"
                set -e # We instruct bash that it CAN again fail on errors
                if [ "${hands_off_ps}" = "" ] ; then
                    if [ -f /usr/bin/killall ] ; then
                        /usr/bin/killall -HUP mDNSResponder
                        logMessage "Flushed the DNS Cache"
                    else
                        logMessage "/usr/bin/killall not present. Not flushing the DNS cache"
                    fi
                else
                    logMessage "Hands Off is running. Not flushing the DNS cache"
                fi
                ;;
        esac
    fi
}

##########################################################################################
trap "" TSTP
trap "" HUP
trap "" INT
export PATH="/bin:/sbin:/usr/sbin:/usr/bin"

readonly SCRIPT_FILE=$0
readonly SCRIPT_LOG_FILE="${SCRIPT_FILE%%.*}.log"

# Quick check - is the configuration there?
if ! scutil -w State:/Network/OpenVPN &>/dev/null -t 1 ; then
    # Configuration isn't there, so we forget it
    logMessage "WARNING: No existing OpenVPN DNS configuration found; not tearing down anything; exiting."
    exit 0
fi

# NOTE: This script does not use any arguments passed to it by OpenVPN, so it doesn't shift Shimo options out of the argument list

# Get info saved by the up script
SHIMO_CONFIG="$( scutil <<-EOF
    open
    show State:/Network/OpenVPN
    quit
EOF
)"

PSID="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*Service :' | sed -e 's/^.*: //g')"
# Don't need: ARG_RESTORE_ON_DNS_RESET="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*RestoreOnDNSReset :' | sed -e 's/^.*: //g')"
# Don't need: ARG_RESTORE_ON_WINS_RESET="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*RestoreOnWINSReset :' | sed -e 's/^.*: //g')"
# Don't need: PROCESS="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*PID :' | sed -e 's/^.*: //g')"
# Don't need: ARG_IGNORE_OPTION_FLAGS="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*IgnoreOptionFlags :' | sed -e 's/^.*: //g')"
ARG_TAP="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*IsTapInterface :' | sed -e 's/^.*: //g')"
ARG_FLUSH_DNS_CACHE="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*FlushDNSCache :' | sed -e 's/^.*: //g')"
ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*ResetPrimaryInterface :' | sed -e 's/^.*: //g')"
bRouteGatewayIsDhcp="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*RouteGatewayIsDhcp :' | sed -e 's/^.*: //g')"
bTapDeviceHasBeenSetNone="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*TapDeviceHasBeenSetNone :' | sed -e 's/^.*: //g')"
bAlsoUsingSetupKeys="$(echo "${SHIMO_CONFIG}" | grep -i '^[[:space:]]*bAlsoUsingSetupKeys :' | sed -e 's/^.*: //g')"

if ${ARG_TAP} ; then
    if [ "$bRouteGatewayIsDhcp" == "true" ]; then
        if [ "$bTapDeviceHasBeenSetNone" == "false" ]; then
            if [ -z "$dev" ]; then
                logMessage "Cannot configure TAP interface for DHCP without \$dev being defined. Device may not have disconnected properly."
            else
                set +e
                ipconfig set "$dev" NONE 2>/dev/null
                set -e
            fi
        fi
    fi
fi

# Issue warning if the primary service ID has changed
set +e # "grep" will return error status (1) if no matches are found, so don't fail if not found
PSID_CURRENT="$( scutil <<-EOF |
    open
    show State:/Network/OpenVPN
    quit
EOF
grep Service | sed -e 's/.*Service : //'
)"
set -e # resume abort on error
if [ "${PSID}" != "${PSID_CURRENT}" ] ; then
    logMessage "Ignoring change of Network Primary Service from ${PSID} to ${PSID_CURRENT}"
fi

# Restore configurations
DNS_OLD="$( scutil <<-EOF
    open
    show State:/Network/OpenVPN/OldDNS
    quit
EOF
)"
SMB_OLD="$( scutil <<-EOF
    open
    show State:/Network/OpenVPN/OldSMB
    quit
EOF
)"
DNS_OLD_SETUP="$( scutil <<-EOF
    open
    show State:/Network/OpenVPN/OldDNSSetup
    quit
EOF
)"
CS_NO_SUCH_KEY="<dictionary> {
    ShimoNoSuchKey : true
}"

if [ "${DNS_OLD}" = "${CS_NO_SUCH_KEY}" ] ; then
    scutil <<-EOF
        open
        remove State:/Network/Service/${PSID}/DNS
        quit
EOF
else
    scutil <<-EOF
        open
        get State:/Network/OpenVPN/OldDNS
        set State:/Network/Service/${PSID}/DNS
        quit
EOF
fi

if [ "${DNS_OLD_SETUP}" = "${CS_NO_SUCH_KEY}" ] ; then
    if ${bAlsoUsingSetupKeys} ; then
        logMessage "DEBUG: Removing 'Setup:' DNS key"
        scutil <<-EOF
            open
            remove Setup:/Network/Service/${PSID}/DNS
            quit
EOF
    else
        logMessage "DEBUG: Not removing 'Setup:' DNS key"
    fi
else
    if ${bAlsoUsingSetupKeys} ; then
        logMessage "DEBUG: Restoring 'Setup:' DNS key"
        scutil <<-EOF
            open
            get State:/Network/OpenVPN/OldDNSSetup
            set Setup:/Network/Service/${PSID}/DNS
            quit
EOF
    else
        logMessage "DEBUG: Not restoring 'Setup:' DNS key"
    fi
fi

if [ "${SMB_OLD}" = "${CS_NO_SUCH_KEY}" ] ; then
    scutil <<-EOF
        open
        remove State:/Network/Service/${PSID}/SMB
        quit
EOF
else
    scutil <<-EOF
        open
        get State:/Network/OpenVPN/OldSMB
        set State:/Network/Service/${PSID}/SMB
        quit
EOF
fi

logMessage "Restored the DNS and SMB configurations"

set +e # "grep" will return error status (1) if no matches are found, so don't fail if not found
new_resolver_contents="`cat /etc/resolv.conf | grep -v '#'`"
set -e # resume abort on error
logMessage "DEBUG:"
logMessage "DEBUG: /etc/resolve = ${new_resolver_contents}"

scutil_dns="$( scutil --dns)"
logMessage "DEBUG:"
logMessage "DEBUG: scutil --dns = ${scutil_dns}"
logMessage "DEBUG:"

flushDNSCache

# Remove our system configuration data
scutil <<-EOF
    open
    remove State:/Network/OpenVPN/OldDNS
    remove State:/Network/OpenVPN/OldSMB
    remove State:/Network/OpenVPN/OldDNSSetup
    remove State:/Network/OpenVPN/DNS
    remove State:/Network/OpenVPN/SMB
    remove State:/Network/OpenVPN
    quit
EOF

if ${ARG_RESET_PRIMARY_INTERFACE_ON_DISCONNECT} ; then

    set +e # "grep" will return error status (1) if no matches are found, so don't fail if not found
    PINTERFACE="$( scutil <<-EOF |
    open
    show State:/Network/Global/IPv4
    quit
EOF
    grep PrimaryInterface | sed -e 's/.*PrimaryInterface : //'
)"
    set -e # resume abort on error

    if [ "${PINTERFACE}" != "" ] ; then
        logMessage "Resetting primary interface '${PINTERFACE}'..."
        /sbin/ifconfig "${PINTERFACE}" down
        sleep 2
        /sbin/ifconfig "${PINTERFACE}" up
    else
        logMessage "Not resetting primary interface because it cannot be found."
    fi

fi

exit 0
